﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace BSF_Utilities
{
    class Fade
    {
        private double _fadeOutTo = 0;
        private double _fadeInTo = 1;
        private double _fadRes = 0.05;
        private bool _closeWhenFinished = false;
        private Form _form;
        private Timer _tmrFadeIn = new Timer();
        private Timer _tmrFadeOut = new Timer();

        private void SetupTimer()
        {
            _tmrFadeIn.Interval = 10;
            _tmrFadeIn.Tick += new EventHandler(FadeIn_Tick);
            _tmrFadeOut.Tick += new EventHandler(FadeOut_Tick);
        }

        /// <summary>
        /// Creates an instance of the fade class
        /// </summary>
        /// <param name="form">the form to be faded</param>
        public Fade(Form form)
        {
            SetupTimer();
            _form = form;
        }

        /// <summary>
        /// Creates an instance of the fade class
        /// </summary>
        /// <param name="form">the form to be faded</param>
        /// <param name="fadeInTo">The opacity value to fade in to, the fade will continue 
        /// until the forms opacity is greater than or equal to this value</param>
        /// <param name="fadeOutTo">The opacity value to fade out to, the fade will continue 
        /// until the forms opacity is less than or equal to this value</param>
        public Fade(Form form, double fadeInTo, double fadeOutTo)
        {
            SetupTimer();
            _form = form;
            _fadeInTo = fadeInTo;
            _fadeOutTo = fadeOutTo;
        }

        /// <summary>
        /// Creates an instance of the fade class
        /// </summary>
        /// <param name="form">the form to be faded</param>
        /// <param name="fadeInTo">The opacity value to fade in to, the fade will continue 
        /// until the forms opacity is greater than or equal to this value</param>
        /// <param name="fadeOutTo">The opacity value to fade out to, the fade will continue 
        /// until the forms opacity is less than or equal to this value</param>
        /// <param name="fadeInSpeed">The speed at which the form will fade in (timer interval)</param>
        /// <param name="fadeOutSpeed">The speed at which the form will fade out (timer interval)</param>
        public Fade(Form form, double fadeInTo, double fadeOutTo, int fadeInSpeed, int fadeOutSpeed)
        {
            SetupTimer();
            _form = form;
            _fadeInTo = fadeInTo;
            _fadeOutTo = fadeOutTo;
            _tmrFadeIn.Interval = fadeInSpeed;
            _tmrFadeOut.Interval = fadeOutSpeed;
        }

        /// <summary>
        /// Initiate fade-in effect
        /// </summary>
        public void StartFadeIn()
        {
            _tmrFadeOut.Stop();
            _tmrFadeIn.Start();
        }

        /// <summary>
        /// Initiate fade-in effect
        /// </summary>
        public void StartFadeOut()
        {
            _tmrFadeIn.Stop();
            _tmrFadeOut.Start();
        }

        private void FadeIn_Tick(object sender, EventArgs e)
        {
            _form.Opacity += _fadRes;

            if (_form.Opacity >= _fadeInTo)
            {
                _tmrFadeIn.Stop();
                _form.Opacity = _fadeInTo;
            }
        }

        private void FadeOut_Tick(object sender, EventArgs e)
        {
            _form.Opacity -= _fadRes;

            if (_form.Opacity <= _fadeOutTo)
            {
                _tmrFadeOut.Stop();
                _form.Opacity = _fadeOutTo;

                if (_closeWhenFinished) _form.Close();
            }
        }

        /// <summary>
        /// Gets or sets the opacity value to fade in to
        /// </summary>
        public double FadeInTo
        {
            get { return _fadeInTo; }
            set 
            {
                if (value < 0)
                {
                    _fadeInTo = 0;
                }
                else if (value > 1)
                {
                    _fadeInTo = 1;
                }
                else
                {
                    _fadeInTo = value;
                }
            }
        }

        /// <summary>
        /// Gets or sets the opacity value to fade out to
        /// </summary>
        public double FadeOutTo
        {
            get { return _fadeOutTo; }
            set 
            {
                if (value < 0)
                {
                    _fadeOutTo = 0;
                }
                else if (value > 1)
                {
                    _fadeOutTo = 1;
                }
                else
                {
                    _fadeOutTo = value;
                }
            }
        }

        /// <summary>
        /// Gets or sets the speed of the form fade in effect (timer interval)
        /// </summary>
        public int FadeInSpeed
        {
            get { return _tmrFadeIn.Interval; }
            set 
            {
                _tmrFadeIn.Interval = value;
            }
        }

        /// <summary>
        /// Gets or sets the speed of the form fade out effect (timer interval)
        /// </summary>
        public int FadeOutSpeed
        {
            get { return _tmrFadeOut.Interval; }
            set 
            {
                _tmrFadeOut.Interval = value;
            }
        }

        /// <summary>
        /// Specifies whether or not the form should close when fade out is finished
        /// </summary>
        public bool CloseWhenFinished
        {
            get { return _closeWhenFinished; }
            set { _closeWhenFinished = value; }
        }
    }
}
