﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Windows.Forms;

namespace BSF_Utilities
{
    public enum Dimension { WIDTH, HEIGHT };

    static class GraphicsHelp
    {
        /// <summary>
        /// Changes the brightness of a color by altering its RGB values
        /// </summary>
        /// <param name="color">The color to be altered</param>
        /// <param name="factor">The amount to change and in what way (Negative is darker)</param>
        /// <returns>Color that has been altered</returns>
        static public Color ChangeColorBrightness(Color color, int factor)
        {
            int R = (color.R + factor > 255) ? 255 : color.R + factor;
            int G = (color.G + factor > 255) ? 255 : color.G + factor;
            int B = (color.B + factor > 255) ? 255 : color.B + factor;

            return Color.FromArgb(R, G, B);
        }

        public static Bitmap DrawControlToBitmap(Control control)
        {
            Bitmap bitmap = new Bitmap(control.Width, control.Height);
            Graphics graphics = Graphics.FromImage(bitmap);
            Rectangle rect = control.RectangleToScreen(control.ClientRectangle);
            graphics.CopyFromScreen(rect.Location, Point.Empty, control.Size);
            return bitmap;
        }

        static public Bitmap CropBitmap(Bitmap bitmap, Rectangle cropArea)
        {
            return bitmap.Clone(cropArea, System.Drawing.Imaging.PixelFormat.Format32bppRgb);
        }

        /// <summary>
        /// Gets the constrained equivilent dimension of another changed dimension by 
        /// finding the difference between the old one and the new one, then returning
        /// the dimension with the difference applied to it.
        /// </summary>
        /// <param name="originalSize">The original size</param>
        /// <param name="dimeToFind">The dimension equivilent to be calculated</param>
        /// <param name="dimeChanged">The changed dimension to find the differnce of</param>
        static public int GetConstrainedDimension(Size originalSize, Dimension dimeToFind, int dimeChanged)
        {
            if (dimeToFind == Dimension.HEIGHT)
            {
                double difference = (double)dimeChanged / (double)originalSize.Width;
                return (int)(originalSize.Height * difference + 0.5);
            }
            else
            {
                double difference = (double)dimeChanged / (double)originalSize.Height;
                return (int)(originalSize.Width * difference + 0.5);
            }
        }

        /// <summary>
        /// Scales a control with a center anchor
        /// </summary>
        /// <param name="Control">The control to be scaled</param>
        /// <param name="scaleFactor">The factor at which to scale</param>
        static public void ScaleAroundCenter(Control control, float scaleFactor)
        { //Note: Depricated overload
            Size lastSize = control.Size;
            control.Size = new Size((int)(control.Width * scaleFactor + 0.5),
                (int)(control.Height * scaleFactor));

            control.Location = new Point(control.Location.X +
                (lastSize.Width - control.Size.Width) / 2,
                control.Location.Y + (lastSize.Height - control.Size.Height) / 2);
        }

        /// <summary>
        /// Scales a control with a center anchor
        /// </summary>
        /// <param name="control">The control to be scaled</param>
        /// <param name="scaleAmount">The amount to scale</param>
        static public void ScaleAroundCenter(Control control, int scaleAmount)
        {
            control.Size = new Size(control.Size.Width + scaleAmount,
                control.Size.Height + scaleAmount);

            control.Location = new Point(control.Location.X - (scaleAmount / 2),
                control.Location.Y - (scaleAmount / 2));
        }

        static public void PaintBorder(Graphics GFX, Pen pen, Control control)
        {
            int startPos = (int)((pen.Width + 0.5) / 2);
            GFX.DrawRectangle(pen, startPos, startPos,
                control.Width - pen.Width, control.Height - pen.Width);
        }
    }
}
