﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Text.RegularExpressions;

/* Make sure you test for content using the HasContent method before using the methods.
 Wrap calls to Save() and the the constructor in a try catch to handle file errors
 */

namespace BSF_Utilities.Helpers
{
    class IniEditor
    {
        public enum GeneralType
        {
            Unknown, // There was no value to process
            Text,    // There were values, non resembled either of the below, so assume a string like type
            Boolean, // A true or false type
            WholeNumber,  // Assume a signed interger
            PreciseNumber // A number with more than zero decimal places
        }

        private List<string> _content = new List<string>();
        private string _filePath;
        private const string _SECTION_PATTERN = @"^\[(?<Section>[^\]]+)\]";
        private const string _PAIR_PATTERN = @"^\s*(?<Key>[^=;#\n]+)=(?<Value>.*)";
        private char _commentCharacter = ';'; // The default comment char will be a semi-colon

        public IniEditor(string filePath)
        {
            _filePath = filePath;
           _content = new List<string>(File.ReadAllLines(filePath));
        }

        public KeyValuePair[] GetPairs()
        {
            List<KeyValuePair> pairs = new List<KeyValuePair>();
            string currentSection = string.Empty;

            foreach (string line in _content)
            {
                // Check for section, I decided to initially check for a section start char for speed
                Match sectionMatch = Regex.Match(line, _SECTION_PATTERN);

                    if (sectionMatch.Success)
                    {
                        currentSection = sectionMatch.Groups["Section"].Value;
                    }

                    Match match = Regex.Match(line, _PAIR_PATTERN, RegexOptions.ExplicitCapture);

                if (match.Success)
                {
                    pairs.Add(new KeyValuePair(match.Groups["Key"].Value, match.Groups["Value"].Value, currentSection));
                }
            }

            return pairs.ToArray();
        }

        /// <summary>
        /// Finds and modifies a value under the given section with the specified key.
        /// </summary>
        /// <param name="sectionName">The name of the section (can include brackets or not)</param>
        /// <param name="keyName">The name of the key to modify</param>
        /// <param name="newValue">The value to set the found or generated key to</param>
        /// <param name="autoCreateSection">When set to true, if section is not found, the section will be inserted at the bottom of the document 
        /// along with the pair that was to be modified</param>
        /// <param name="autoCreatePair">When set to true, if the section is found and the target pair is not, then the pair will 
        /// be inserted at the top of the section</param>
        /// <returns>Returns true, if pre-existing key was found</returns>
        public bool ModifyValue(string sectionName, string keyName, string newValue, bool autoCreateSection, bool autoCreatePair)
        {
            sectionName = sectionName.TrimStart('[');
            sectionName = sectionName.TrimEnd(']');

            bool sectionFound = false;
            bool pairFound = false;
            int sectionIndex = -1;

            for (int i = 0; i < _content.Count; i++)
            {
                Match match = Regex.Match(_content[i], _SECTION_PATTERN);

                if (match.Success && match.Groups["Section"].Value == sectionName)
                {
                    sectionFound = true;
                    sectionIndex = i;

                    for (int i2 = i + 1; i2 < _content.Count; i2++)
                    {
                        if (Regex.IsMatch(_content[i2], _SECTION_PATTERN))
                        {
                            break;
                        }
                        else
                        {
                            Match pairMatch = Regex.Match(_content[i2], _PAIR_PATTERN, RegexOptions.ExplicitCapture);

                            if (pairMatch.Success && pairMatch.Groups["Key"].Value == keyName)
                            {
                               int index = pairMatch.Groups["Value"].Index;
                               int length = pairMatch.Groups["Value"].Length;

                               _content[i2] = _content[i2].Remove(index, length);
                               _content[i2] = _content[i2].Insert(index, newValue);
                               pairFound = true;
                            }
                        }
                    }

                    break;
                }
            }

            if (!sectionFound && autoCreateSection)
            {
                // Only add new line if needed
                if (_content[_content.Count - 1].Trim() != string.Empty)
                {
                    _content.Add(string.Empty);
                }
                _content.Add("[" + sectionName + "]");
                _content.Add(keyName + "=" + newValue);
            }
            else if (sectionFound && !pairFound && autoCreatePair)
            {
                // Bounds check?
                _content.Insert(sectionIndex + 1, keyName + "=" + newValue);
            }

            return pairFound;
        }

        /// <summary>
        /// Deletes a key value pair that matches the specifed key, regardless of where it is in the file
        /// </summary>
        /// <returns>Returns false, if nothing has been deleted</returns>
        public bool RemoveAnyPair(string keyName)
        {
            bool hasDeleted = false;

            for (int i = 0; i < _content.Count; i++)
            {
                Match match = Regex.Match(_content[i], _PAIR_PATTERN, RegexOptions.ExplicitCapture);

                if (match.Success && match.Groups["Key"].Value == keyName)
                {
                    _content.RemoveAt(i);
                    i--;
                    hasDeleted = true;
                }
            }

            return hasDeleted;
        }

        /// <summary>
        /// Deletes a pair in the specified section
        /// </summary>
        /// <param name="sectionName">The name of the section the pairs has to reside in</param>
        /// <param name="keyName">The name of the key</param>
        /// <returns>Returns true, if a pair has been found and removed</returns>
        public bool RemovePair(string sectionName, string keyName)
        {
            sectionName = sectionName.TrimStart('[');
            sectionName = sectionName.TrimEnd(']');

            for (int i = 0; i < _content.Count; i++)
            {
                Match sectionMatch = Regex.Match(_content[i], _SECTION_PATTERN);

                if (sectionMatch.Success && sectionMatch.Groups["Section"].Value == sectionName)
                {
                    if (i + 1 < _content.Count)
                    {
                        i++;

                        for (int i2 = i; i2 < _content.Count; i2++)
                        {
                            if (!Regex.IsMatch(_content[i2], _SECTION_PATTERN, RegexOptions.ExplicitCapture))
                            {
                                Match keyMatch = Regex.Match(_content[i2], _PAIR_PATTERN, RegexOptions.ExplicitCapture);

                                if (keyMatch.Success)
                                {
                                    string key = keyMatch.Groups["Key"].Value;

                                    if (keyName == key)
                                    {
                                        _content.RemoveAt(i2);
                                        return true;
                                    }
                                }
                            }
                            else
                            {
                                // Encountered end of section before finding pair
                                return false;
                            }
                        }
                    }
                }
            }

            return false;
        }

        /// <summary>
        /// Removes all comments (starts with a semi-colon)
        /// </summary>
        public void RemoveAllComments()
        {
            for (int i = 0; i < _content.Count; i++)
            {
                if (_content[i].Length > 0 && _content[i][0].Equals(_commentCharacter))
                {
                    _content.RemoveAt(i);
                    i--;
                }
            }
        }

        /// <summary>
        /// Indents all pairs with a tab character
        /// </summary>
        public void IndentAllPairs()
        {
            for (int i = 0; i < _content.Count; i++)
            {
                Match match = Regex.Match(_content[i], _PAIR_PATTERN, RegexOptions.ExplicitCapture);

                if (match.Success)
                {
                    _content[i] = _content[i].Trim();
                    _content[i] = '\t' + _content[i];
                }
            }
        }

        /// <summary>
        /// Indents all pairs with the specfied amount of spaces
        /// </summary>
        public void IndentAllPairs(int spaces)
        {
            StringBuilder SB = new StringBuilder();

            for (int i = 0; i < spaces; i++)
            {
                SB.Append(' ');
            }

            string strSpaces = SB.ToString();

            for (int i = 0; i < _content.Count; i++)
            {
                Match match = Regex.Match(_content[i], _PAIR_PATTERN, RegexOptions.ExplicitCapture);

                if (match.Success)
                {
                    _content[i] = _content[i].Trim();
                    _content[i] = strSpaces + _content[i];
                }
            }
        }

        /// <summary>
        /// Unindents all pairs with a tab character
        /// </summary>
        public void UnindentAllPairs()
        {
            for (int i = 0; i < _content.Count; i++)
            {
                Match match = Regex.Match(_content[i], _PAIR_PATTERN, RegexOptions.ExplicitCapture);

                if (match.Success)
                {
                    _content[i] = _content[i].Trim();
                }
            }
        }

        /// <summary>
        /// Deletes all keys that have no value set (values that have nothing or just whitepace)
        /// </summary>
        public void RemoveEmptyKeys()
        {
            for (int i = 0; i < _content.Count; i++)
            {
                Match match = Regex.Match(_content[i], _PAIR_PATTERN, RegexOptions.ExplicitCapture);

                if (match.Success && match.Groups["Value"].Value.Trim().Equals(string.Empty))
                {
                    _content.RemoveAt(i);
                    i--;
                }
            }
        }

        /// <summary>
        /// Removes all blank lines and inserts blank lines above all
        /// the section headers unless the header is on line index 0
        /// </summary>
        public void FormatBlankLines()
        {
            if (_content.Count.Equals(0))
            {
                return;
            }

            // Remove all blank lines
            for (int i = 0; i < _content.Count; i++)
            {
                if (_content[i].Trim().Equals(string.Empty))
                {
                    _content.RemoveAt(i);
                }
            }

            // Insert blank line above section header, if section header not the first line
            for (int i = 1; i < _content.Count; i++)
            {
                if (Regex.IsMatch(_content[i], _SECTION_PATTERN))
                {
                    _content.Insert(i, string.Empty);
                    i++; // Iterate over section header or we will encounter it again
                }
            }
        }

        /// <summary>
        /// Gets the value of the first occurrence of the speified key
        /// </summary>
        /// <returns>Returns null if nothing found</returns>
        public string GetValueOfAnyKey(string keyName)
        {
            for (int i = 0; i < _content.Count; i++)
            {
                Match match = Regex.Match(_content[i], _PAIR_PATTERN, RegexOptions.ExplicitCapture);

                if (match.Success && match.Groups["Key"].Value == keyName)
                {
                    return match.Groups["Value"].Value;
                }
            }

            return null;
        }

        /// <summary>
        /// Gets the value of the first occurrence of the speified key in the specified section
        /// </summary>
        /// <returns>Returns null if nothing found</returns>
        public string GetValueOfKey(string keyName, string sectionName)
        {
            sectionName = sectionName.TrimStart('[');
            sectionName = sectionName.TrimEnd(']');

            int sectionIndex = -1;

            for (int i = 0; i < _content.Count; i++)
            {
                Match match = Regex.Match(_content[i], _SECTION_PATTERN, RegexOptions.ExplicitCapture);

                if (match.Success && match.Groups["Section"].Value == sectionName)
                {
                    sectionIndex = i;
                }
            }

            if (sectionIndex == -1)
            {
                return null;
            }
            else
            {
                for (int i = sectionIndex + 1; i < _content.Count; i++)
                {
                    if (Regex.IsMatch(_content[i], _SECTION_PATTERN))
                    {
                        break;
                    }
                    else
                    {
                        Match match = Regex.Match(_content[i], _PAIR_PATTERN);

                        if (match.Success && match.Groups["Key"].Value == keyName)
                        {
                            return match.Groups["Value"].Value;
                        }
                    }
                }


            }

            return null;
        }

        /// <summary>
        /// Removes the section header and all values under it until another section header appears
        /// </summary>
        /// <returns>returns true, if section has successfully been removed</returns>
        public bool RemoveSection(string sectionName)
        {
            sectionName = sectionName.TrimStart('[');
            sectionName = sectionName.TrimEnd(']');

            int removeStart = -1;

            for (int i = 0; i < _content.Count; i++) 
            {
                Match match = Regex.Match(_content[i], _SECTION_PATTERN, RegexOptions.ExplicitCapture);

                if (match.Success && match.Groups["Section"].Value.Equals(sectionName))
                {
                    removeStart = i;
                    break; // Our section has been found, break
                }
            }

            // could not find section, return false to indicate failure
            if (removeStart == -1)
            {
                return false;
            }
            else
            {
                for (int i = removeStart + 1; i < _content.Count; i++)
                {
                    Match match = Regex.Match(_content[i], _SECTION_PATTERN, RegexOptions.ExplicitCapture);

                    if (match.Success)
                    {
                        int length = i - removeStart - 1;
                        _content.RemoveRange(removeStart, length);

                        if (_content[removeStart].Trim().Equals(string.Empty))
                        {
                            _content.RemoveAt(removeStart);
                        }

                        return true;
                    }
                    else if (i.Equals(_content.Count - 1))
                    {
                        // Remove end minus start gets the length
                        int length = (_content.Count - 1) - removeStart;
                        _content.RemoveRange(removeStart, length);

                        if (_content[removeStart].Trim().Equals(string.Empty))
                        {
                            _content.RemoveAt(removeStart);
                        }

                        return true;
                    }
                }
            }

            return false;
        }

        /// <summary>
        /// Saves the changes to file
        /// </summary>
        public void Save()
        {
             File.WriteAllLines(_filePath, _content);
        }

        /// <summary>
        /// Gets the best suited type of the value of the specified key and the specified section
        /// </summary>
        public GeneralType GetGeneralType(string keyName, string sectionName)
        {
            string value = GetValueOfKey(keyName, sectionName).Trim();
            int num = 0;
            float f = 0;

            if (value.Length.Equals(0))
            {
                return GeneralType.Unknown;
            }
            else if (value.ToLower().Equals("true") || value.ToLower().Equals("false"))
            {
                return GeneralType.Boolean;
            }
            else if (int.TryParse(value, out num))
            {
                return GeneralType.WholeNumber;
            }
            else if (float.TryParse(value, out f))
            {
                return GeneralType.PreciseNumber;
            }
            else
            {
                return GeneralType.Text;
            }
        }

        /// <summary>
        /// Gets the best suited type of the value of the specified key
        /// </summary>
        public GeneralType GetGeneralTypeOfAny(string keyName)
        {
            string value = GetValueOfAnyKey(keyName).Trim();
            int num = 0;
            float f = 0;

            if (value.Length.Equals(0))
            {
                return GeneralType.Unknown;
            }
            else if (value.ToLower().Equals("true") || value.ToLower().Equals("false"))
            {
                return GeneralType.Boolean;
            }
            else if (int.TryParse(value, out num))
            {
                return GeneralType.WholeNumber;
            }
            else if (float.TryParse(value, out f))
            {
                return GeneralType.PreciseNumber;
            }
            else
            {
                return GeneralType.Text;
            }
        }

        #region Properties
        /// <summary>
        /// Determines whether or not content is avaliable
        /// </summary>
        public bool HasContent
        {
            get { return _content.Count > 0; }
        }

        /// <summary>
        /// The path of the ini file loaded
        /// </summary>
        public string FilePath
        {
            get { return _filePath; }
        }

        /// <summary>
        /// The character recognized as the comment start, acceptable characters are the
        /// pound symbol (#) or the semi-colon (;)
        /// </summary>
        public char CommentCharacter
        {
            get { return _commentCharacter; }
            set { _commentCharacter = value; }
        }

        /// <summary>
        /// Gets the amount of comments. Only supports full-line comments and uses the comment character set
        /// </summary>
        public int CommentCount
        {
            get
            {
                int count = 0;

                for (int i = 0; i < _content.Count; i++)
                {
                    string line = _content[i].TrimStart(' ');

                    if (line.Length > 0 && line[0].Equals(_commentCharacter))
                    {
                        count++;
                    }
                }

                return count;
            }
        }

        /// <summary>
        /// Gets the count fo the key value pairs (does not included commented)
        /// </summary>
        public int PairCount
        {
            get
            {
                int count = 0;

                for (int i = 0; i < _content.Count; i++)
                {
                    if (Regex.IsMatch(_content[i], _PAIR_PATTERN, RegexOptions.ExplicitCapture))
                    {
                        count++;
                    }
                }

                return count;
            }
        }

        /// <summary>
        /// Gets the section count (does not included commented)
        /// </summary>
        public int SectionCount
        {
            get
            {
                int count = 0;

                for (int i = 0; i < _content.Count; i++)
                {
                    if (Regex.IsMatch(_content[i], _SECTION_PATTERN, RegexOptions.ExplicitCapture))
                    {
                        count++;
                    }
                }

                return count;
            }
        }
        #endregion
    }

    class KeyValuePair
    {
        private string _keyName, _value, _section;

        public KeyValuePair(string keyName, string keyVal, string section)
        {
            _keyName = keyName;
            _value = keyVal;
            _section = section;
        }

        public string KeyName
        {
            get { return _keyName; }
            set { _keyName = value; }
        }

        public string Value
        {
            get { return _value; }
            set { _value = value; }
        }

        public string Section
        {
            get { return _section; }
            set { _section = value;  }
        }
    }
}
