﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.ComponentModel;
using System.Drawing.Drawing2D;

namespace BSF_Code_Library.Helpers
{
    public enum LineIntersectType { HorizOnTop, VertOnTop, Subtract };
    /// <summary>
    /// For best results, set the doublebufferd value to true on control
    /// </summary>
    class MouseTraker
    {
        private BufferedGraphics _bufferedGraphics;
        private BufferedGraphicsContext _bufferContext = BufferedGraphicsManager.Current;
        private Pen _horizPen = new Pen(Color.Blue, 3);
        private Pen _vertPen = new Pen(Color.CornflowerBlue, 3);
        private Control _control;
        private LineIntersectType _intersectType = LineIntersectType.Subtract;
        private bool _showHorizLine = true, _showVertLine = true;
        private bool _autoHide = true, _antiAlias;

        public MouseTraker(Control control)
        {
            _control = control;
            UpdateGraphicsBuffer();

            control.MouseMove += new MouseEventHandler(control_MouseMove);
            control.MouseLeave += new EventHandler(_control_MouseLeave);
            control.SizeChanged += new EventHandler(control_SizeChanged);
            control.Paint += new PaintEventHandler(control_Paint);
        }

        /// <summary>
        /// Updates the drawing surface and backbuffer as well as configures the backbuffer
        /// </summary>
        private void UpdateGraphicsBuffer()
        {
            _bufferContext.MaximumBuffer = _control.ClientSize;
            _bufferedGraphics = _bufferContext.Allocate(_control.CreateGraphics(), _control.ClientRectangle);

            if (_antiAlias)
            {
                _bufferedGraphics.Graphics.SmoothingMode = SmoothingMode.AntiAlias;
            }
            else
            {
                _bufferedGraphics.Graphics.SmoothingMode = SmoothingMode.None;
            }
        }

        /// <summary>
        /// Sets the width of both horizontal line and vertical line
        /// </summary>
        public void SetLineStroke(float stroke)
        {
            _horizPen.Width = stroke;
            _vertPen.Width = stroke;
        }

        #region Control Events
        private void control_MouseMove(object sender, MouseEventArgs e)
        {
            if (_showHorizLine || _showVertLine) // Don't do anything if lines are not shown
            {
                _bufferedGraphics.Graphics.Clear(_control.BackColor);

                if (_showHorizLine)
                    _bufferedGraphics.Graphics.DrawLine(_horizPen, 0, e.Y, _control.Width, e.Y);

                if (_showVertLine)
                    _bufferedGraphics.Graphics.DrawLine(_vertPen, e.X, 0, e.X, _control.Height);

                switch (_intersectType)
                {
                    case LineIntersectType.Subtract:
                        {
                            float xPos = e.X - _vertPen.Width / 2f;
                            float yPos = e.Y - _vertPen.Width / 2f;

                            _bufferedGraphics.Graphics.FillRectangle(new SolidBrush(_control.BackColor),
                                xPos, yPos, _vertPen.Width, _vertPen.Width);
                        }
                        break;

                    case LineIntersectType.HorizOnTop:
                        {
                            float xPos = e.X - _vertPen.Width / 2f;
                            float yPos = e.Y - _vertPen.Width / 2f;

                            _bufferedGraphics.Graphics.FillRectangle(_horizPen.Brush,
                                xPos, yPos, _vertPen.Width, _vertPen.Width);
                        }
                        break;
                }

                // Make sure to invalidate all children or children will get left over paint on their borders
                _control.Invalidate(true);
            }
        }

        private void _control_MouseLeave(object sender, EventArgs e)
        {
            // If autoHide enabled and atleast one of the lines is showing
            if (_autoHide && (_showHorizLine || _showVertLine))
            {
                _bufferedGraphics.Graphics.Clear(_control.BackColor);
                _control.Invalidate(true);
            }
        }

        private void control_SizeChanged(object sender, EventArgs e)
        {
            UpdateGraphicsBuffer();
        }

        private void control_Paint(object sender, PaintEventArgs e)
        {
            _bufferedGraphics.Render(e.Graphics);
        }
        #endregion

        #region Properties
        [Browsable(false)]
        public Pen VerticalPen
        {
            get { return _vertPen; }
            set { _vertPen = value; }
        }

        [Browsable(false)]
        public Pen HorizontalPen
        {
            get { return _horizPen; }
            set { _horizPen = value; }
        }

        [DefaultValue(true)]
        public bool AutoHide
        {
            get { return _autoHide; }
            set { _autoHide = value; }
        }

        [DefaultValue(true)]
        public bool ShowVerticalLine
        {
            get { return _showVertLine; }
            set { _showVertLine = value; }
        }

        [DefaultValue(true)]
        public bool ShowHorizontalLine
        {
            get { return _showHorizLine; }
            set { _showHorizLine = value; }
        }

        [DefaultValue(false)]
        public bool AntiAlias
        {
            get { return _antiAlias; }
            set
            {
                _antiAlias = value;
                UpdateGraphicsBuffer();
            }
        }

        /// <summary>
        /// Gets the recommended control styles for control in which this class paints on
        /// </summary>
        public static ControlStyles RecommendStyle
        {
            get
            {
                return ControlStyles.AllPaintingInWmPaint | ControlStyles.UserPaint
                    | ControlStyles.OptimizedDoubleBuffer;
            }
        }

        [DefaultValue(typeof(LineIntersectType), "Subtract")]
        public LineIntersectType IntersectType
        {
            get { return _intersectType; }
            set { _intersectType = value; }
        }
        #endregion
    }
}
