﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.ComponentModel;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using System.Diagnostics;

/* This class works by scanning a specified page for version info.
 * It then compares the version info found with the apps assembly version.
 * A message box is shown accordingly, if a newer version is available a link
 * to the version will be sent to the users default web browser.
 * The version info can be any where in the pages HTML content. Sytax is as follows:
 * 
 * ApplicationName=[ver]1.0.0.0b[ver]
 * The application name should be what the line "Application.ProductName" returns in your app.
 * Where the version info is, all 4 digits should be present seperated by a period.
 * You do not need to specify the app status. b = beta, a = alpha otherwise assume its stable.
 * All of the messages are exposed via properties so you may adapt to your liking or language(s).
 * Anything under version 1.0.0.0 will be deemed alpha and the message will show accordingly
 * 
 * Example of update page: http://ableopus.weebly.com/updates.html
 * */

namespace BSF_Code_Library.Helpers
{
    class UpdateChecker
    {
        private string _downloadLink, _updatePage; // Page to scan for versioning

        private string _downloadErrorMsg =
            "Could not retrieve data from update page!" +
             "\nYou can check manually on developers site";

        private string _newVersionMsg =
            "A newer version is available, click OK to download.";

        private string _betaDownloadMsg =
            "\n\nNote: The version you are downloading is in beta and may have a few bugs.";

        private string _alphaDownloadMsg =
            "\n\nNote: The version you are downloading is in alpha and may be a bit unstable.";

        private string _upToDateMsg = "Your version of {0} is up to date!";
        private string _downloadLinkErrorMsg = "Could not navigate to download location";
        private string _scanErrorMsg = "Unable to find valid update info, sorry for the inconvenience!";

        public UpdateChecker(string downloadLink, string updatePage)
        {
            _downloadLink = downloadLink;
            _updatePage = updatePage;
        }

        public void BeginChecking()
        {
            WebClient webClient = new WebClient();

            webClient.DownloadStringCompleted
                += new DownloadStringCompletedEventHandler(webClient_DownloadStringCompleted);

            try
            {
                webClient.DownloadStringAsync(new Uri(_updatePage));
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void webClient_DownloadStringCompleted(object sender, DownloadStringCompletedEventArgs e)
        {
            if (e.Error != null || string.IsNullOrEmpty(e.Result))
            {
                MessageBox.Show(_downloadErrorMsg);
                return;
            }

            // ex: Notepad Enhanced=[ver]7.4.6.0b[ver]

            string pattern = Application.ProductName +
                @"=\[ver\](?<version>\d\.\d\.\d\.\d)(?<status>|a|b)\[ver\]";

            Regex regex = new Regex(pattern, RegexOptions.IgnoreCase);
            Match match = regex.Match(e.Result);

            string finding = match.Groups["version"].Value;
            string status = match.Groups["status"].Value;

            if (string.IsNullOrEmpty(finding))
            {
                ShowError();
                return;
            }

            int latestVersion = VersionToInt(finding);
            int appVersion = VersionToInt(Application.ProductVersion);

            if (latestVersion == 0)
            {
                ShowError();
                return;
            }

            CheckVersions(appVersion, latestVersion, status);
        }

        private void CheckVersions(int appVersion, int latestVersion, string status)
        {
            if (appVersion < latestVersion)
            {
                string msg = _newVersionMsg;

                if (status == "b")
                {
                    msg += _betaDownloadMsg;
                }
                else if (status == "a" || latestVersion < 1000)
                {
                    msg += _alphaDownloadMsg;
                }

                DialogResult result = MessageBox.Show(msg, Application.ProductName, 
                    MessageBoxButtons.OKCancel, MessageBoxIcon.Asterisk);

                if (result == DialogResult.OK)
                {
                    GoToDownloadLocation();
                }
            }
            else
            {
                string message = string.Format(_upToDateMsg, Application.ProductName);

                DialogResult result = MessageBox.Show(message, Application.ProductName,
                    MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }
        }

        private void GoToDownloadLocation()
        {
            try
            {
                Process.Start(_downloadLink);
            }
            catch
            {
                MessageBox.Show(_downloadLinkErrorMsg, Application.ProductName,
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private static int VersionToInt(string version)
        {
            version = version.Replace(".", string.Empty);
            int versionNum = 0;
            int.TryParse(version, out versionNum);
            return versionNum;
        }

        private void ShowError()
        {
            MessageBox.Show(_scanErrorMsg, Application.ProductName,
                MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
        }

        #region Properties
        /// <summary>
        /// The link to the latest version of you program which coinsides with 
        /// version found when scanning the update page
        /// </summary>
        public string DownloadLink
        {
            get { return _downloadLink; }
            set { _downloadLink = value; }
        }

        /// <summary>
        /// The URL of the update web-page to scan for version info
        /// </summary>
        public string UpdatePageToScan
        {
            get { return _updatePage; }
            set { _updatePage = value; }
        }

        /// <summary>
        /// The message to appear when a error occurs when downloading update page
        /// </summary>
        public string DownloadErrorMessage
        {
            get { return _downloadErrorMsg; }
            set { _downloadErrorMsg = value; }
        }

        /// <summary>
        /// The message to appear when a new version of this program is available
        /// </summary>
        public string NewVersionMessage
        {
            get { return _newVersionMsg; }
            set { _newVersionMsg = value; }
        }

        /// <summary>
        /// The string to be added to the "New Version Message" when 
        /// the download is marked as beta
        /// </summary>
        public string BetaDownloadMessage
        {
            get { return _betaDownloadMsg; }
            set { _betaDownloadMsg = value; }
        }

        /// <summary>
        /// The string to be added to the "New Version Message" when 
        /// the download is marked as alpha
        /// </summary>
        public string AlphaDownloadMessage
        {
            get { return _alphaDownloadMsg; }
            set { _alphaDownloadMsg = value; }
        }

        /// <summary>
        /// The message to appaer when this program is up to date
        /// </summary>
        public string UpToDateMessage
        {
            get { return _upToDateMsg; }
            set { _upToDateMsg = value; }
        }

        /// <summary>
        /// The message to appear when an error occurs trying 
        /// navigate to the download location
        /// </summary>
        public string DownloadLinkError
        {
            get { return _downloadLinkErrorMsg; }
            set { _downloadLinkErrorMsg = value; }
        }

        /// <summary>
        /// The message to appear when information could not be parsed
        /// from the update page. Ex: from improperly formatted versioning info
        /// </summary>
        public string ScanErrorMessage
        {
            get { return _scanErrorMsg; }
            set { _scanErrorMsg = value; }
        }
        #endregion
    }
}
