﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Drawing;

namespace BSF_Code_Library.Helpers
{
    struct Vertex
    {
        private int _x, _y, _z;

        /// <summary>
        /// Creates a vertex object
        /// </summary>
        /// <param name="x">The X position of the vertex</param>
        /// <param name="y">The Y position of the vertex</param>
        /// <param name="z">The Z position of the vertex</param>
        public Vertex(int x, int y, int z)
        {
            _x = x;
            _y = y;
            _z = z;
        }

        #region Operator overloading
        public static bool operator ==(Vertex v1, Vertex v2)
        {
            bool isXEqual = v1._x == v2._x;
            bool isYEqual = v1._y == v2._y;
            bool isZEqual = v1._z == v2._z;

            return (isXEqual && isZEqual && isYEqual);
        }

        public static bool operator !=(Vertex v1, Vertex v2)
        {
            bool isXEqual = v1._x == v2._x;
            bool isYEqual = v1._y == v2._y;
            bool isZEqual = v1._z == v2._z;

            return !(isXEqual && isZEqual && isYEqual);
        }

        public static bool operator <(Vertex v1, Vertex v2)
        {
            int total1 = v1._x + v1._y + v1._z;
            int total2 = v2._x + v2._y + v2._z;

            return (total1 < total2);
        }

        public static bool operator >(Vertex v1, Vertex v2)
        {
            int total1 = v1._x + v1._y + v1._z;
            int total2 = v2._x + v2._y + v2._z;

            return (total1 > total2);
        }

        public static bool operator <=(Vertex v1, Vertex v2)
        {
            int total1 = v1._x + v1._y + v1._z;
            int total2 = v2._x + v2._y + v2._z;

            return (total1 <= total2);
        }

        public static bool operator >=(Vertex v1, Vertex v2)
        {
            int total1 = v1._x + v1._y + v1._z;
            int total2 = v2._x + v2._y + v2._z;

            return (total1 >= total2);
        }

        public static Vertex operator *(Vertex v1, Vertex v2)
        {
            Vertex v3 = new Vertex
                (v1._x * v2._x, v1._y * v2._y, v1._z * v2._z);
            return v3;
        }

        public static Vertex operator ++(Vertex v1)
        {
            v1._x++;
            v1._y++;
            v1._z++;
            return v1;
        }

        public static Vertex operator --(Vertex v1)
        {
            v1._x--;
            v1._y--;
            v1._z--;
            return v1;
        }

        public static Vertex operator /(Vertex v1, Vertex v2)
        {
            Vertex v3 = new Vertex
                (v1._x / v2._x, v1._y / v2._y, v1._z / v2._z);
            return v3;
        }

        public override int GetHashCode()
        {
            return
            _x.GetHashCode() ^
            _y.GetHashCode() ^
            _z.GetHashCode();
        }

        public override bool Equals(object obj)
        {
            if (!(obj is Vertex)) return false;

            Vertex v1 = (Vertex)obj;

            bool isXEqual = v1._x == this._x;
            bool isYEqual = v1._y == this._y;
            bool isZEqual = v1._z == this._z;

            return (isXEqual && isZEqual && isYEqual);
        }
        #endregion

        #region Properties
        [DefaultValue(0)]
        [Category("Axii")]
        [Description("The X axis of the vertex")]
        [DisplayName("X Position")]
        /// <summary>
        /// Gets or sets the X position of the vertex
        /// </summary>
        public int X
        {
            get { return _x; }
            set { _x = value; }
        }

        [DefaultValue(0)]
        [Category("Axii")]
        [Description("The Y axis of the vertex")]
        [DisplayName("Y Position")]
        /// <summary>
        /// Gets or sets the Y position of the vertex
        /// </summary>
        public int Y
        {
            get { return _y; }
            set { _y = value; }
        }

        [DefaultValue(0)]
        [Category("Axii")]
        [Description("The Z axis of the vertex")]
        [DisplayName("Z Position")]
        /// <summary>
        /// Gets or sets the Z position of the vertex
        /// </summary>
        public int Z
        {
            get { return _z; }
            set { _z = value; }
        }
        #endregion

        /// <summary>
        /// Resets the vertex to its original position
        /// </summary>
        public void ResetToOrigin()
        {
            _x = _y = _z = 0;
        }

        public override string ToString()
        {
            return String.Format("{{X={0}, Y={1}, Z={2}}}", _x, _y, _z);
        }
    }

    struct VertexColored
    {
        private int _x;
        private int _y;
        private int _z;
        private Color _color;

        /// <summary>
        /// Creates a vertex object
        /// </summary>
        /// <param name="x">The X position of the vertex</param>
        /// <param name="y">The Y position of the vertex</param>
        /// <param name="z">The Z position of the vertex</param>
        public VertexColored(int x, int y, int z, Color color)
        {
            _x = x;
            _y = y;
            _z = z;
            _color = color;
        }

        #region Operator overloading
        public static bool operator ==(VertexColored v1, VertexColored v2)
        {
            bool isXEqual = v1._x == v2._x;
            bool isYEqual = v1._y == v2._y;
            bool isZEqual = v1._z == v2._z;

            return (isXEqual && isZEqual && isYEqual);
        }

        public static bool operator !=(VertexColored v1, VertexColored v2)
        {
            bool isXEqual = v1._x == v2._x;
            bool isYEqual = v1._y == v2._y;
            bool isZEqual = v1._z == v2._z;

            return !(isXEqual && isZEqual && isYEqual);
        }

        public static bool operator <(VertexColored v1, VertexColored v2)
        {
            int total1 = v1._x + v1._y + v1._z;
            int total2 = v2._x + v2._y + v2._z;

            return (total1 < total2);
        }

        public static bool operator >(VertexColored v1, VertexColored v2)
        {
            int total1 = v1._x + v1._y + v1._z;
            int total2 = v2._x + v2._y + v2._z;

            return (total1 > total2);
        }

        public static bool operator <=(VertexColored v1, VertexColored v2)
        {
            int total1 = v1._x + v1._y + v1._z;
            int total2 = v2._x + v2._y + v2._z;

            return (total1 <= total2);
        }

        public static bool operator >=(VertexColored v1, VertexColored v2)
        {
            int total1 = v1._x + v1._y + v1._z;
            int total2 = v2._x + v2._y + v2._z;

            return (total1 >= total2);
        }

        public static Vertex operator *(VertexColored v1, VertexColored v2)
        {
            Vertex v3 = new Vertex
                (v1._x * v2._x, v1._y * v2._y, v1._z * v2._z);
            return v3;
        }

        public static VertexColored operator ++(VertexColored v1)
        {
            v1._x++;
            v1._y++;
            v1._z++;
            return v1;
        }

        public static VertexColored operator --(VertexColored v1)
        {
            v1._x--;
            v1._y--;
            v1._z--;
            return v1;
        }

        public static VertexColored operator /(VertexColored v1, VertexColored v2)
        {
            VertexColored v3 = new VertexColored
                (v1._x / v2._x, v1._y / v2._y, v1._z / v2._z, v2._color);
            return v3;
        }

        public override int GetHashCode()
        {
            return
            _x.GetHashCode() ^
            _y.GetHashCode() ^
            _z.GetHashCode();
        }

        public override bool Equals(object obj)
        {
            if (!(obj is Vertex)) return false;

            VertexColored v1 = (VertexColored)obj;

            bool isXEqual = v1._x == this._x;
            bool isYEqual = v1._y == this._y;
            bool isZEqual = v1._z == this._z;

            return (isXEqual && isZEqual && isYEqual);
        }
        #endregion

        #region Properties
        [DefaultValue(0)]
        [Category("Axii")]
        [Description("The X axis of the vertex")]
        [DisplayName("X Position")]
        /// <summary>
        /// Gets or sets the X position of the vertex
        /// </summary>
        public int X
        {
            get { return _x; }
            set { _x = value; }
        }

        [DefaultValue(0)]
        [Category("Axii")]
        [Description("The Y axis of the vertex")]
        [DisplayName("Y Position")]
        /// <summary>
        /// Gets or sets the Y position of the vertex
        /// </summary>
        public int Y
        {
            get { return _y; }
            set { _y = value; }
        }

        [DefaultValue(0)]
        [Category("Axii")]
        [Description("The Z axis of the vertex")]
        [DisplayName("Z Position")]
        /// <summary>
        /// Gets or sets the Z position of the vertex
        /// </summary>
        public int Z
        {
            get { return _z; }
            set { _z = value; }
        }

        [Category("Appearance")]
        [Description("The color of the vertex")]
        [DisplayName("Vertex Color")]
        /// <summary>
        /// Gets or sets the color of the vertex
        /// </summary>
        public Color VertexColor
        {
            get { return _color; }
            set { _color = value; }
        }
        #endregion

        /// <summary>
        /// Resets the vertex to its original position
        /// </summary>
        public void ResetToOrigin()
        {
            _x = _y = _z = 0;
        }

        public override string ToString()
        {
            return String.Format("{{X={0}, Y={1}, Z={2}}}", _x, _y, _z);
        }
    }
}
